/*
 * $Id$
 */

package org.codehaus.mojo.taglist.output.io.xpp3;

  //---------------------------------/
 //- Imported classes and packages -/
//---------------------------------/

import java.io.Writer;
import java.text.DateFormat;
import java.util.Iterator;
import java.util.Locale;
import org.codehaus.mojo.taglist.output.TagListXMLComment;
import org.codehaus.mojo.taglist.output.TagListXMLFile;
import org.codehaus.mojo.taglist.output.TagListXMLReport;
import org.codehaus.mojo.taglist.output.TagListXMLTag;
import org.codehaus.plexus.util.xml.pull.MXSerializer;
import org.codehaus.plexus.util.xml.pull.XmlSerializer;

/**
 * Class TaglistOutputXpp3Writer.
 * 
 * @version $Revision$ $Date$
 */
public class TaglistOutputXpp3Writer {


      //--------------------------/
     //- Class/Member Variables -/
    //--------------------------/

    /**
     * Field NAMESPACE.
     */
    private String NAMESPACE;


      //-----------/
     //- Methods -/
    //-----------/

    /**
     * Method write.
     * 
     * @param writer
     * @param tagListXMLReport
     * @throws java.io.IOException
     */
    public void write( Writer writer, TagListXMLReport tagListXMLReport )
        throws java.io.IOException
    {
        XmlSerializer serializer = new MXSerializer();
        serializer.setProperty( "http://xmlpull.org/v1/doc/properties.html#serializer-indentation", "  " );
        serializer.setProperty( "http://xmlpull.org/v1/doc/properties.html#serializer-line-separator", "\n" );
        serializer.setOutput( writer );
        serializer.startDocument( tagListXMLReport.getModelEncoding(), null );
        writeTagListXMLReport( tagListXMLReport, "report", serializer );
        serializer.endDocument();
    } //-- void write( Writer, TagListXMLReport ) 

    /**
     * Method writeTagListXMLComment.
     * 
     * @param tagListXMLComment
     * @param serializer
     * @param tagName
     * @throws java.io.IOException
     */
    private void writeTagListXMLComment( TagListXMLComment tagListXMLComment, String tagName, XmlSerializer serializer )
        throws java.io.IOException
    {
        if ( tagListXMLComment != null )
        {
            serializer.startTag( NAMESPACE, tagName );
            if ( tagListXMLComment.getLineNumber() != null )
            {
                serializer.startTag( NAMESPACE, "lineNumber" ).text( tagListXMLComment.getLineNumber() ).endTag( NAMESPACE, "lineNumber" );
            }
            if ( tagListXMLComment.getComment() != null )
            {
                serializer.startTag( NAMESPACE, "comment" ).text( tagListXMLComment.getComment() ).endTag( NAMESPACE, "comment" );
            }
            serializer.endTag( NAMESPACE, tagName );
        }
    } //-- void writeTagListXMLComment( TagListXMLComment, String, XmlSerializer ) 

    /**
     * Method writeTagListXMLFile.
     * 
     * @param tagListXMLFile
     * @param serializer
     * @param tagName
     * @throws java.io.IOException
     */
    private void writeTagListXMLFile( TagListXMLFile tagListXMLFile, String tagName, XmlSerializer serializer )
        throws java.io.IOException
    {
        if ( tagListXMLFile != null )
        {
            serializer.startTag( NAMESPACE, tagName );
            if ( tagListXMLFile.getName() != null )
            {
                serializer.attribute( NAMESPACE, "name", tagListXMLFile.getName() );
            }
            if ( tagListXMLFile.getCount() != null )
            {
                serializer.attribute( NAMESPACE, "count", tagListXMLFile.getCount() );
            }
            if ( tagListXMLFile.getComments() != null && tagListXMLFile.getComments().size() > 0 )
            {
                serializer.startTag( NAMESPACE, "comments" );
                for ( Iterator iter = tagListXMLFile.getComments().iterator(); iter.hasNext(); )
                {
                    TagListXMLComment o = (TagListXMLComment) iter.next();
                    writeTagListXMLComment( o, "comment", serializer );
                }
                serializer.endTag( NAMESPACE, "comments" );
            }
            serializer.endTag( NAMESPACE, tagName );
        }
    } //-- void writeTagListXMLFile( TagListXMLFile, String, XmlSerializer ) 

    /**
     * Method writeTagListXMLReport.
     * 
     * @param tagListXMLReport
     * @param serializer
     * @param tagName
     * @throws java.io.IOException
     */
    private void writeTagListXMLReport( TagListXMLReport tagListXMLReport, String tagName, XmlSerializer serializer )
        throws java.io.IOException
    {
        if ( tagListXMLReport != null )
        {
            serializer.startTag( NAMESPACE, tagName );
            if ( tagListXMLReport.getTags() != null && tagListXMLReport.getTags().size() > 0 )
            {
                serializer.startTag( NAMESPACE, "tags" );
                for ( Iterator iter = tagListXMLReport.getTags().iterator(); iter.hasNext(); )
                {
                    TagListXMLTag o = (TagListXMLTag) iter.next();
                    writeTagListXMLTag( o, "tag", serializer );
                }
                serializer.endTag( NAMESPACE, "tags" );
            }
            serializer.endTag( NAMESPACE, tagName );
        }
    } //-- void writeTagListXMLReport( TagListXMLReport, String, XmlSerializer ) 

    /**
     * Method writeTagListXMLTag.
     * 
     * @param tagListXMLTag
     * @param serializer
     * @param tagName
     * @throws java.io.IOException
     */
    private void writeTagListXMLTag( TagListXMLTag tagListXMLTag, String tagName, XmlSerializer serializer )
        throws java.io.IOException
    {
        if ( tagListXMLTag != null )
        {
            serializer.startTag( NAMESPACE, tagName );
            if ( tagListXMLTag.getName() != null )
            {
                serializer.attribute( NAMESPACE, "name", tagListXMLTag.getName() );
            }
            if ( tagListXMLTag.getCount() != null )
            {
                serializer.attribute( NAMESPACE, "count", tagListXMLTag.getCount() );
            }
            if ( tagListXMLTag.getFiles() != null && tagListXMLTag.getFiles().size() > 0 )
            {
                serializer.startTag( NAMESPACE, "files" );
                for ( Iterator iter = tagListXMLTag.getFiles().iterator(); iter.hasNext(); )
                {
                    TagListXMLFile o = (TagListXMLFile) iter.next();
                    writeTagListXMLFile( o, "file", serializer );
                }
                serializer.endTag( NAMESPACE, "files" );
            }
            serializer.endTag( NAMESPACE, tagName );
        }
    } //-- void writeTagListXMLTag( TagListXMLTag, String, XmlSerializer ) 


}
